@@ -139,6 +139,14 @@ signals:
     void passwordRequest(bool includingUsername = false);
     void outputErrorMessage(const QString &message);
 
+    /**
+     * When we connect/disconnect/reconnect/etc., this signal will be emitted.
+     *
+     * @param status            Is the client connected?
+     * @param details           A sentence describing what happened.
+     */
+    void clientStateChanged(RemoteView::RemoteStatus status, const QString &details);
+
 public slots:
     void mouseEvent(int x, int y, int buttonMask);
     void keyEvent(int key, bool pressed);
@@ -148,15 +156,26 @@ protected:
     void run();
 
 private:
-    static void setClientColorDepth(rfbClient *cl, ColorDepth cd);
+    void setClientColorDepth(rfbClient *cl, ColorDepth cd);
     void setColorDepth(ColorDepth colorDepth);
-    //these static methods are callback functions for libvncclient
-    static rfbBool newclient(rfbClient *cl);
-    static void updatefb(rfbClient *cl, int x, int y, int w, int h);
-    static void cuttext(rfbClient *cl, const char *text, int textlen);
-    static char* passwdHandler(rfbClient *cl);
-    static rfbCredential* credentialHandler(rfbClient *cl, int credentialType);
-    static void outputHandler(const char *format, ...);
+
+    // These static methods are callback functions for libvncclient. Each
+    // of them calls back into the corresponding member function via some
+    // TLS-based logic.
+    static rfbBool newclientStatic(rfbClient *cl);
+    static void updatefbStatic(rfbClient *cl, int x, int y, int w, int h);
+    static void cuttextStatic(rfbClient *cl, const char *text, int textlen);
+    static char *passwdHandlerStatic(rfbClient *cl);
+    static rfbCredential *credentialHandlerStatic(rfbClient *cl, int credentialType);
+    static void outputHandlerStatic(const char *format, ...);
+
+    // Member functions corresponding to the above static methods.
+    rfbBool newclient();
+    void updatefb(int x, int y, int w, int h);
+    void cuttext(const char *text, int textlen);
+    char *passwdHandler();
+    rfbCredential *credentialHandler(int credentialType);
+    void outputHandler(const char *format, ...);
 
     QImage m_image;
     rfbClient *cl;
@@ -169,11 +188,48 @@ private:
     ColorDepth m_colorDepth;
     QQueue<ClientEvent* > m_eventQueue;
     //color table for 8bit indexed colors
-    static QVector<QRgb> m_colorTable;
+    QVector<QRgb> m_colorTable;
+    QString outputErrorMessageString;
 
     volatile bool m_stopped;
     volatile bool m_passwordError;
 
+    /**
+     * Connection keepalive/reconnection support.
+     */
+    struct {
+        /**
+         * Number of seconds between probes. If zero, we will not attempt
+         * to enable it.
+         */
+        int intervalSeconds;
+        /**
+         * Number of failed probes required to recognise a disconnect.
+         */
+        int failedProbes;
+        /**
+         * Was keepalive successfully set?
+         */
+        bool set;
+        /**
+         * Did keepalive detect a disconnect?
+         */
+        volatile bool failed;
+    } m_keepalive;
+
+    // Initialise the VNC client library object.
+    bool clientCreate(bool reinitialising);
+
+    // Uninitialise the VNC client library object.
+    void clientDestroy();
+
+    // Turn on keepalive support.
+    void clientSetKeepalive();
+
+    // Record a state change.
+    void clientStateChange(RemoteView::RemoteStatus status, const QString &details);
+    QString m_previousDetails;
+
 private slots:
     void checkOutputErrorMessage();
 };
